#!/usr/bin/perl

if(scalar(@ARGV) < 2)
{
  print <<EOF;

Index Generator
by Jonah Cohen <ComAsYuAre\@aol.com>
http://jonah.ticalc.org

Usage:
  $0 source_dir dest_dir

EOF
  exit(0);
}

open(TOP, "$ARGV[0]/top.tex") || die("Can't open $ARGV[0]/top.tex\n");
while(<TOP>)
{
  $previous = $1 if /^[^%]*\\include\s*\{([^\}]+)\}/;
}

open(INDEX, ".index_lst") || die("Can't open .index_lst\n");;
while(<INDEX>)
{
  chomp;
  /(.*)\t(.*)/;
  $index{$1} = $2;
}
close(INDEX);

open(OUTPUT, ">$ARGV[1]/dex.htm") || die("Can't write to $ARGV[1]/dex.htm\n");

$b = $/;
$/ = undef;

open(HEADER, "html_header.html") || die("Can't read from html_header.html\n");
$header = <HEADER>;
close(HEADER);

open(HEADERFOOTER, "html_header_footer.html") || die("Can't read from html_header_footer.html\n");
$headerfooter = <HEADERFOOTER>;
close(HEADERFOOTER);

$/ = $b;

$header =~ s/\$title/Index/g;

$headerfooter =~ s/next.png/blank.png/g;
$headerfooter =~ s/<a href="\$next[^>]*>(<[^<]*)<\/a>/$1/;
$headerfooter =~ s/\$previous/$previous/;

print "Creating Index\n";

#write HTML header
print OUTPUT <<EOF;
$header
$headerfooter

<h1>Index</h1>

<table border="0" cellspacing="0" cellpadding="20">
<tr>
<td valign="top" width="50%">
<font size=2>
EOF

#sort items alphabetically (case-insensitive)
@items = sort {uc($a) cmp uc($b)} keys %index;

$i = 0;
$n = int((scalar(@items)+1)/2);
foreach (@items)
{
  $i++;

  if(s/\|([^\{]*)\{\s*([^\}]*)\}?/... $1 $2/)
  {
    print OUTPUT "$_<br>\n";
    next;
  }

  @links = split /,/,$index{$_};

  s/^.+\!(.+)$/&nbsp; &nbsp; &nbsp; $1/;
  s/\!//g;
  print OUTPUT "$_... ";

  while ($_ = shift @links)
  {
    /-/;
    print OUTPUT "<a href=$'>$`</a>";
    if(scalar(@links)>0)
    {
      print OUTPUT ", ";
    }
    else
    {
      print OUTPUT "\n";
    }
  }

  if($i == $n)
  {
    print OUTPUT <<EOF
</font>
</td>
<td valign="top" width="50%">
<font size=2>
EOF
  }
  else
  {
    print OUTPUT "<br>";
  }
}

#write HTML footer
print OUTPUT <<EOF;
</font>
</tr>
</table>
$headerfooter
</body>
</html>
EOF

close(OUTPUT);
