#!/usr/bin/perl

if(scalar(@ARGV) < 3)
{
  print <<EOF;

Latex-to-HTML converter
by Jonah Cohen <ComAsYuAre\@aol.com>
http://jonah.ticalc.org

Usage:
  $0 filename source_dir dest_dir

Converts from source_dir/filename.tex to dest_dir/filename.htm

EOF
  exit(0);
}

open(INPUT, "$ARGV[1]/$ARGV[0].tex") || die("Can't read from $ARGV[1]/$ARGV[0].htm\n");
@lines=<INPUT>;
close(INPUT);
push @lines,"";  #so it stops parsing

$chapnum = $ARGV[0];
$chapnum =~ s/[^\d]//g;
$chap = $chapnum;
$chap =~ s/^0//;
$sec = 0; #starting section number

#for regex
$brace = "\\s*(([^\\{\\}]*(\\{[^\\}]*\\})?)*)\\}";
$braces = "\\s*\\{" . $brace;

#control characters
%cchars = ("\\_","#95","\\#","#35","<","lt","\\<","lt",">","gt","\\>","gt","\\&","amp","\\\%","#37","\\{","#123","\\}","#125","\\\$","#36","\\copyright","copy");
$cchars = "\\\\_|\\\\#|\\\\?<|\\\\?>|\\\\&|\\\\\\\%|\\\\\\{|\\\\\\}|\\\\\\\$|\\\\copyright";

#converted back later
%ccharsb = ("#95","_","#35","#","amp","&","#37","\%","#123","{","#125","}","#36","\$");
$ccharsb = "#95|#35|amp|#37|#123|#125|#36";

open(OUTPUT, ">$ARGV[2]/$ARGV[0].htm") || die("Can't write to $ARGV[2]/$ARGV[0].htm\n");


push @files,"index";
open(TOP, "$ARGV[1]/top.tex") || die("Can't read from $ARGV[1]/top.tex\n");
while(<TOP>)
{
  push @files,$1 if /^[^%]*\\include\{([^\}]+)\}/;
}
close(TOP);
push @files,"dex";

open(LABELS, ".labels_lst");
while(<LABELS>)
{
  chomp;
  /(.*)\t(.*)\t(.*)/;
  $labels{$1} = $2;
  $labellinks{$1} = $3;
}
close(LABELS);

open(INDEX, ".index_lst");
while(<INDEX>)
{
  chomp;
  /(.*)\t(.*)/;
  $index{$1} = $2;
}
close(INDEX);

$b = undef $/;
open(HEADER, "html_header.html") || die("Can't read from html_header.html\n");
$header = <HEADER>;
close(HEADER);

open(HEADERFOOTER, "html_header_footer.html") || die("Can't read html header\n");
$headerfooter = <HEADERFOOTER>;
close(HEADERFOOTER);

$/ = $b;

print "Converting $ARGV[0].tex to $ARGV[0].htm\n";

#find previous and next files
for($i=0; $i<scalar(@files); $i++)
{
  if($ARGV[0] =~ $files[$i])
  {
    $previous = $files[$i-1];
    $next = $files[$i+1];
    last;
  }
}

#find title
foreach (@lines)
{
  if(/^[^%]*\\chapter$braces/)
  {
    $chapname = $1;
    last;
  }
}

if($ARGV[0] =~ /ch/)
{
  $title = "Chapter $chap: $chapname";
  $chapter = "\n<h2>Chapter $chap</h2>\n";
}
elsif($ARGV[0] =~ /ap/)
{
  $title = "Appendix " . pack('c',64+$chap) . ": $chapname";
  $chapter = "\n<h2>Appendix " . pack('c',64+$chap) . "</h2>\n";
}
else
{
  $title = $chapname;
  $chapter = "";
}

$header =~ s/\$title/$title/;
$headerfooter =~ s/\$previous/$previous/g;
$headerfooter =~ s/\$next/$next/g;

print OUTPUT <<EOF;
$header
$headerfooter
$chapter
<h1>$chapname</h1>
EOF

$p = "";  #paragraph string

#formatting: 0 = normal, 1 = straight text, 2 = math, 3 = table
$c = 0;

foreach (@lines)
{
  chomp;

  #replace control characters
  s/($cchars)/&$cchars{$1}\;/g;
  s/``|''/"/g unless $c==1;

  #erase comments and extraneous whitespace
  s/%.*// unless $c==1;
  s/\s*$//;

  if(/\\begin\s*\{verbatim\}/)
  {
    print OUTPUT &revert(&latex($p)) . "\n";
    $p = "";
    print OUTPUT "<pre>\n";
    $c = 1;
    next;
  }
  elsif(/\\end\s*\{verbatim\}/)
  {
    print OUTPUT "</pre>\n\n";
    $c = 0;
    next;
  }
  elsif(/\\begin\s*\{(eqnarray\*|equation)\}/)
  {
    print OUTPUT &revert(&latex($p)) . "\n\n";
    $p = "";
    print OUTPUT "<table align=center><tr><td><tt>\n";
    $c = 2;
    next;
  }
  elsif(/\\end\s*\{(eqnarray\*|equation)\}/)
  {
    print OUTPUT "</tt></td></tr></table>\n\n";
    $c = 0;
    next;
  }
  elsif(/\\begin\s*\{(table|tabular)\}/)
  {
    print OUTPUT &revert(&latex($p)) . "\n\n";
    $p = "";
    print OUTPUT "<table align=center>\n";
    $c = 3;
    next;
  }
  elsif(/\\end\s*\{(table|tabular)\}/)
  {
    print OUTPUT "</table>\n";
    $c = 0;
    next;
  }

  if($c == 3) #table
  {
    $_ = &latex($_);
    s/<\/?p>//g;
    s/\\(begin|end)\{tabbing\}//g;
    s/.*\\kill.*//g;
    next unless /./;
    $_ = join "",map "<td style=\"padding-right: 8;\">$_</td>",split /\s+&\s+/;
    print OUTPUT "<tr>" . &revert($_) . "<\/tr>\n";
  }
  elsif($c == 2) #math
  {
    print OUTPUT &revert(&math($_,1)) . "\n";
  }
  elsif($c == 1) #<pre> tag
  {
    print OUTPUT &revert($_) . "\n";
  }
  else #regular latex
  {
    #erase leading whitespace
    s/^\s*//;

    #keep going if blank line with mismatching curly braces
    $_ = "\n<p>" if /^$/ && ($p =~ s/\{/$&/g) != ($p =~ s/\}/$&/g);

    #append line to current paragraph
    $p .= "$_ ";

    #done if we're on a blank line
    if(/^\s*$/)
    {
      #display filtered paragraph
      print OUTPUT &revert(&latex($p)) . "\n\n" unless $p =~ /^\s*$/;

      #reset paragraph string
      $p = "";
    }
  }
}

#if there's any footnotes, write javascript function
if($fn)
{
  $/ = undef;
  open(STYLE,"style.html");
  $style = <STYLE>;
  close(STYLE);

  $style =~ s/["\n]//gm;

  print OUTPUT <<EOF;
<script language="Javascript">
<!--
function fn(str)
{
  var w = window.open("","newwin","height=200,width=200");
  w.document.write("<html><head><title>Note</title>$style<\\/head><body bgcolor=white>" + str + "<p><a href=\\"javascript:self.close();\\"><small><font color=blue><b>Close window</b></small></font></a></p></body></html>");
}
//-->
</script>

EOF
}

#write HTML footer
print OUTPUT <<EOF;
<p>
$headerfooter
</body>
</html>
EOF

close(OUTPUT);

sub latex
{
  $_ = $_[0];

  #taken care of in parser
  s/\\index$braces//g;
  s/\\label$braces//g;

  #substitute references to labels
  s/(\w*)~\\ref$braces/<a href="$labellinks{$2}">$1 $labels{$2}<\/a>/g;

  #remove unsupported latex tags
  s/\\pagebreak//g;
  s/\\vspace$braces//g;
  s/\\noindent//g;

  #remove whitespace
  s/^\s*//;
  s/\s*$//;

  #hyperlinks
  s/\{\\tt\s+((ht|f)tp:\/\/\S+)\s*\}/<a href="$1">$1<\/a>/g;
  s/\{\\tt\s+(\S+@\S+)\s*\}/<a href="mailto:$1">$1<\/a>/g;

  #basic formatting tags
  s/\{\\bf$brace/<b>$1<\/b>/g;
  s/\\textbf$braces/<b>$1<\/b>/g;
  s/\{\\em$brace/<i>$1<\/i>/g;
  s/\{\\tt$brace/<code>$1<\/code>/g;

  #footnotes
  #$fn += s/\\footnote$braces/" <a href=\"javascript:fn(\'" . (join "\\\'",split /\'/,$1) . "\')\">* Note<\/a>"/ge;
  $fn += s/\\footnote$braces/" <a href=\"javascript:fn(\'" . &footnote($1) . "\')\">* Note<\/a>"/ge;

  #math syntax
  $_ = &math($_,0);

  #tags that have their own paragraph marks
  if
  (
    #headers
    s/\\chapter$braces//g +
    s/\\section$braces/&section($1)/ge +
    s/\\subsection$braces/<h4>$1<\/h4>/g +

    #list items
    s/\\item\s*\[([^:\]]*):?\]\s*([^\\]*)/<dt>$1<\/dt>\n<dd>$2<\/dd>/g +
    s/\\item\s*([^\[\\][^\\]*)/<li>$1<\/li>/g +

    #unordered lists
    s/\\begin\s*\{itemize\}/<ul>/g +
    s/\\end\s*\{itemize\}/<\/ul>/g +

    #ordered lists
    s/\\begin\s*\{enumerate\}/<ol>/g +
    s/\\end\s*\{enumerate\}/<\/ol>/g +

    #definition lists
    s/\\begin\s*\{description\}/<dl>/g +
    s/\\end\s*\{description\}/<\/dl>/g +

    #blockquotes
    s/\\begin\s*\{quote\}/<p style="margin-left: 30px; margin-right: 30px;">/g +
    s/\\end\s*\{quote\}/<\/p>/g +

    #images
    s/\\centerline\s*\{\\psfig\s*\{figure=([^\.]*)[^\}]*\}\s*\}/<p align="center"><img src="images\/$1.png"><\/p>/g +

    #centered text
    s/\\centerline$braces/<b>$1<\/b>/g
  )
  {
    return $_;
  }
  elsif($_ ne "")
  {
    return "<p>$_</p>";
  }
}

sub section
{
  $_ = $_[0];
  if($ARGV[0] =~ /ch/)
  {
    $sec++;
    return "<a name=$sec><\/a><br>\n<h3>$chap.$sec $_<\/h3>";
  }
  else
  {
    return "<br>\n<h3>$_<\/h3>";
  }
}

sub math
{
  $_ = $_[0];

  s/\\\$/&dollar;/g;
  s/\s*\\\\\s*/<br>/g;

  if($_[1])
  {
    $inmath = 1;
    @a = ($_);
  }
  else
  {
    $inmath = 0;
    @a = split /\$/;
  }

  $str = "";
  foreach (@a)
  {
    if($inmath)
    {
      s/\&\&//g;
      s/&\s+=\s+&/=/g;
      s/^\s*//;
      s/\s*$//;

      #superscript and subscript
      s/\_$braces/<sub>$1<\/sub>/g;
      s/\_(.)/<sub>$1<\/sub>/g;
      s/\^$braces/<sup>$1<\/sup>/g;
      s/\^(.)/<sup>$1<\/sup>/g;

      #math functions
      s/\\sqrt$braces/sqrt($1)/g;
      s/\\(log|ln)/$1/g;
      s/\\frac$braces$braces/($1)\/($4)/g;
      s/\\((arc)?(sin|cos|tan))/$1/g;
      s/\\mathcal$braces\s*/<big><i>$1<\/i><\/big>/g;
      s/\\mapsto/-\&gt;/g;

      #special chars
      s/\\backslash/&#92;/g;
      s/\\(pi|theta)/$1/g;
      s/\\cdots/&middot;&middot;&middot;/g;
      s/\\cdot/&middot;/g;

      #backslashes
      #s/\\/&#92;/g;
      s/\\//g;

      $_ = "<tt>$_</tt>" unless $_[1];
      $str .= $_ if /./;
    }
    else
    {
      $str .= $_;
    }
    $inmath = 1-$inmath;
  }

  $_ = $str;

  s/&dollar;/\$/g;
  s/&#95;/_/g;

  return $_;
}

sub revert
{
  my $s = pop;
  #characters that were changed for parsing only
  $s =~ s/&($ccharsb);/$ccharsb{$1}/gm;
  return $s;
}

sub footnote
{
  my $s = pop;
  $s =~ s/'/\\'/g;
  return $s;
}
