#!/usr/bin/perl

if(scalar(@ARGV) < 1)
{
  print <<EOF;

Latex Parser
by Jonah Cohen <ComAsYuAre\@aol.com>
http://jonah.ticalc.org

Usage:
  $0 file source_dir/

Parses info from source_dir/file.tex

EOF
  exit(0);
}

open(INPUT, "$ARGV[1]/$ARGV[0].tex") || die("Can't read from $ARGV[1]/$ARGV[0].tex\n");
@lines=<INPUT>;
close(INPUT);

$chap = $ARGV[0];
$chap =~ s/^.*?(\d+)$/$1/;
$chapnum = $chap + 0;
$sec = 0;

#for regex
$braces = "\\s*\\{([^\\}]+)*\\}";

#control characters
%cchars = ("\\_","#95","\\#","#35","<","lt","\\<","lt",">","gt","\\>","gt","\\&","amp","&","amp","``","quot","''","quot","\\\%","#37","\\{","#123","\\}","#125","\\\$","#36","\\copyright","copy");
$cchars = "\\\\_|\\\\#|\\\\?<|\\\\?>|\\\\&|&|``|''|\\\\\\\%|\\\\\\{|\\\\\\}|\\\\\\\$|\\\\copyright";

open(LABELS, ".labels_lst");
while(<LABELS>)
{
  chomp;
  /(.*)\t(.*)\t(.*)/;
  $labels{$1} = $2;
  $labellinks{$1} = $3;
}
close(LABELS);

open(LABELS, ">.labels_lst") || die("Can't write to .labels_lst\n");

open(INDEX, ".index_lst");
while(<INDEX>)
{
  chomp;
  /(.*)\t(.*)/;
  $index{$1} = $2;
}
close(INDEX);

open(INDEX, ">.index_lst")  || die("Can't write to .index_lst.\n");

print "Parsing $ARGV[0].tex\n";

foreach (@lines)
{
  s/$cchars/&$cchars{$&};/g;

  $sec += /\\section$braces/;

  #look for indexes
  while(s/\\index$braces//)
  {
    $i = $1;
    $i =~ s/^\s*//;
    $i =~ s/\s*$//;
    $indexlink = "$ARGV[0].htm";
    if($ARGV[0] =~ /ch/)
    {
      $index = $chapnum;
      if($sec)
      {
        $index .= ".$sec";
        $indexlink .= "#$sec";
      }
    }
    else
    {
      $index = "Appendix " . pack('c',64+$chap);
    }
    unless($i =~ /^$/)
    {
      %indices = ();
      foreach (split /,/,$index{$i})
      {
        /-/;
        $indices{$`} = $';
      }
      $indices{$index} = $indexlink;
      $index{$i} = join ",",map "$_-$indices{$_}",sort {&sec($a) <=> &sec($b)} keys %indices;
    }
  }

  #look for labels
  while(s/\\label$braces//)
  {
    $l = $1;
    $l =~ s/^\s*//;
    $l =~ s/\s*$//;
    if($ARGV[0] =~ /ch/)
    {
      $labels{$l} = 0+$chap;
      $labellinks{$l} = "chap$chap.htm";
      if($sec)
      {
        $labels{$l} .= ".$sec";
        $labellinks{$l} .= "#$sec";
      }
    }
    else
    {
      $labels{$l} = pack('c',64+$chap);
      $labellinks{$l} = "app$chap.htm";
    }
  }
}

foreach (keys %labels)
{
  print LABELS "$_\t$labels{$_}\t$labellinks{$_}\n";
}
close(LABELS);

foreach (keys %index)
{
  print INDEX "$_\t$index{$_}\n";
}
close(INDEX);

sub sec()
{
  $_ = pop;
  unless(/\./)
  {
    $_ = 100 + unpack('c',$_);
  }
  else
  {
    s/\.(\d)$/.0$1/;
  }
  return $_;
}
