#!/usr/bin/perl

if(scalar(@ARGV) < 2)
{
  print <<EOF;

Syntax Highlighter
by Jonah Cohen <ComAsYuAre\@aol.com>
http://jonah.ticalc.org

Usage:
  $0 lang source.html [dest.html]

EOF
  exit(0);
}

open(LANG, "$ARGV[0].lang") || die("Can't open $ARGV[0].lang\n");
($case,$n) = split /\s+/,<LANG>;
while($n--)
{
  $_ = <LANG>;
  s/[\n\r]//g;
  @a = split;
  $a[1] =~ s/([\/\.\*\{\}\&\?\+\[\]])/\\$1/g;
  $comments{$a[0]} = $a[1];
}
$comments = join "|",keys %comments;
$comments =~ s/([\/\.\*\{\}\&\?\+\[\]])/\\$1/g;

$/ = undef;

$keywords = <LANG>;
$keywords =~ s/[\s\n\r]+/|/g;
$keywords =~ s/(^\||\|$)//g;
close(LANG);

open(STYLE, "style.html") || print("Can't open style.html\n");
$style = <STYLE>;
close(STYLE);

open(INPUT, $ARGV[1]) || die("Can't open $ARGV[1]\n");
@lines = split /<\/?(code|pre)[^>]*>[\n\r]*/im,<INPUT>;
close(INPUT);

$out = pop;
open(OUTPUT, ">$out") || die("Can't write to $out\n");

print "Syntax highlighting $ARGV[1]\n";

$incode = -1;
while(@lines)
{
  $_ = shift @lines;
  $incode = ($incode+1)%4;
  unless($incode)
  {
    s/<\/head>/$style$&/mi;
    print OUTPUT;
  }
  elsif($incode == 2)
  {
    if(/^\s*((ht|f)tp:\/\/\S+|\S+\@\S+\.\S+)\s*$/)
    {
      $link = $1;
      $link = "<a href=\"" . ((/\@/)?"mailto:":"") . "$link\">$link</a>";
      $link = "<p>$link</p>" if shift(@lines) =~ /pre/i;
      $incode++;

      print OUTPUT $link;
      next;
    }

    print OUTPUT "<span class=code>";

    s/<\/?tt>//gm;
    s/</&lt;/gm;
    s/>/&gt;/gm;
    s/\n/\n<br>/gm;
    while(s/^(.*?)\t/&spaces($1)/gme) {}
    s/  /&nbsp; /gm;
    s/  / &nbsp;/gm;


  startover: # :-)

    @comments = ();
    @line = ();

    while(/(^|[^&])($comments)/m)
    {
      push @comments,$2;
      push @line,$`.$1;
      $_ = $';

      /($comments{$2})/m || /()\z/m;
      push @comments,$1;
      push @line,$`;
      $_ = $';
    }
    push @line,$_;
    $incomment = 1;

    while(@line)
    {
      $_ = shift @line;
      $incomment = !$incomment;
      if($incomment)
      {
        print OUTPUT "<span class=comment>" . shift(@comments) . $_ . shift(@comments) . "</span>";
      }
      else
      {
        @quotes = ();
        @quoteline = ();
        while(/((^|[^\\])(\\\\)*)(["'`])/m)
        {
          push @quotes,$4;
          push @quoteline,$`.$1;
          $_ = $';

          $q = $4;

          if(/((^|[^\\])(\\\\)*)($q)/m)
          {
            push @quotes,$4;
            push @quoteline,$`.$1;
            $_ = $';
            next;
          }

          until(/((^|[^\\])(\\\\)*)($q)/m || scalar(@comments)==0)
          {
            $_ .= shift(@comments) . shift(@line);
          }

          unless(/((^|[^\\])(\\\\)*)($q)/m)
          {
            $_ = pop(@quoteline) . pop(@quotes) . $_;
            last;
          }

          push @quotes,$4;
          push @quoteline,$`.$1;

          $_ = $';
          $_ .= shift(@comments) . shift(@line) while @comments || @line;

          &dispquotes;
          goto startover;
        }
        push @quoteline,$_;
        &dispquotes;
      }
    }
    print OUTPUT "</span>";
  }
}

close(OUTPUT);

sub dispquotes
{
  $inquote = 1;
  foreach (@quoteline)
  {
    $inquote = !$inquote;
    if($inquote)
    {
      print OUTPUT "<span class=quote>" . shift(@quotes) . $_ . shift(@quotes) . "</span>";
    }
    else
    {
      if($case eq "S")
      {
        s/\b($keywords)\b((\s|&nbsp;|<br>)+\b($keywords)\b)*/<span class=keyword>$&<\/span>/gm;
        s/(<span class=keyword>def<\/span>\s+)(\w+)/$1<span class=function>$2<\/span>/gm;
      }
      else
      {
        s/\b($keywords)\b((\s|&nbsp;|<br>|\n)+\b($keywords)\b)*/<span class=keyword>$&<\/span>/gmi;
      }
      print OUTPUT;
    }
  }
}

sub spaces()
{
  $s = pop;
  $_ = $s;
  s/&[^;]*;/_/g;
  return $s . (" " x (4-length($_)%4));
}
