#!/usr/bin/perl

if(scalar(@ARGV) < 2)
{
  print <<EOF;

Table of Contents Generator
by Jonah Cohen <ComAsYuAre\@aol.com>
http://jonah.ticalc.org

Usage:
  $0 source_dir dest_dir

Parses info from file.tex

EOF
  exit(0);
}

#for regex
$braces = "\\s*\\{\\s*(([^\\{\\}]*(\\{[^\\}]*\\})?)*)\\}";

open(TOP, "$ARGV[0]/top.tex") || die("Can't open $ARGV[0]/top.tex.\n");
@top=<TOP>;
close(TOP);

foreach (@top)
{
  $title = $1 if /^[^%]*\\title$braces/;
  $author = $1 if /^[^%]*\\author$braces/;
  $lang = ucfirst $1 if /([\w+#]+)\s*Version/i;
}

$title =~ s/(\w+)/ucfirst $1/ge;
$title =~ s/\\\w*\s*//g;

open(OUTPUT, ">$ARGV[1]/index.htm") || die("Can't write to $ARGV[1]/index.htm.\n");

print "Creating Table of Contents\n";

print OUTPUT <<EOF;
<html>

<head>
  <title>$title</title>
  <style type="text/css">

body {background-color: #fffacd; text-align: center}
a {text-decoration: none; color: #a0522d; font-size: 11pt; 
font-weight: bold; font-family: Verdana}
a:hover {text-decoration: underline}
a:active {text-decoration: underline}
a.h2 {font-size: 15pt; font-family: Times}
a.h3 {font-size: 12pt; font-family: Times}
h1 {font-size: 22pt; font-family: Times}
p.h2 {font-size: 15pt; font-family: Times}
table {border: 0; margin-left: auto; margin-right: auto}
td {background-color: #b0e0e6}
th {font-size: 12pt; font-family: Times}

</style>
</head>

<body>
  <h1>$title</h1>
  <h2>$lang Version</h2>
  <p class=h2>by $author</p>
  
<table cellspacing=6 cellpadding=7 align=center>
  <tr>
    <th>Table of Contents</th>
  </tr>
EOF

foreach (@top)
{
  next unless /^[^%]*\\include$braces/;

  $file = $1;
  $n = $1;
  $n =~ s/[^\d]//g;
  $n =~ s/^0//;

  #get title
  open(FILE, "$ARGV[0]/$file.tex") || next;
  foreach (<FILE>)
  {
    if(/^[^%]*\\chapter$braces/)
    {
      $title = $1;
      last;
    }
  }
  close(FILE);

  if($file =~ /ch/)
  {
    $title = "Chapter $n: $title";
  }
  elsif($file =~ /ap/)
  {
    $title = "Appendix " . pack('c',64+$n) . ": $title";
  }

  print OUTPUT <<EOF; 
  <tr>
    <td><a href="$file.htm">$title</a></td>
  </tr>
EOF
}

print OUTPUT <<EOF;
  <tr>
    <td><a href="dex.htm">Index</a></td>
  </tr>
</table>

</body>
</html>
EOF

close(OUTPUT);
